"""Array data-type implementations (abstraction points for GL array types"""
import ctypes
import OpenGL
assert OpenGL
from OpenGL.raw.GL import _types
from OpenGL import plugins
from OpenGL.arrays import formathandler, _arrayconstants as GL_1_1
from OpenGL import logs
_log = logs.getLog( 'OpenGL.arrays.arraydatatype' )
try:
    unicode
except NameError:
    unicode = str

from OpenGL import acceleratesupport
ADT = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.arraydatatype import ArrayDatatype as ADT
    except ImportError as err:
        _log.warning(
            "Unable to load ArrayDatatype accelerator from OpenGL_accelerate"
        )
if ADT is None:
    # Python-coded version
    class HandlerRegistry( dict ):
        GENERIC_OUTPUT_PREFERENCES = ['numpy','ctypesarrays']
        def __init__( self, plugin_match ):
            self.match = plugin_match
            self.output_handler = None 
            self.preferredOutput = None
            self.all_output_handlers = []
        def __call__( self, value ):
            """Lookup of handler for given value"""
            try:
                typ = value.__class__
            except AttributeError:
                typ = type(value)
            handler = self.get( typ )
            if not handler:
                if hasattr( typ, '__mro__' ):
                    for base in typ.__mro__:
                        handler = self.get( base )
                        if not handler:
                            handler = self.match( base )
                            if handler:
                                handler = handler.load()
                                if handler:
                                    handler = handler()
                        if handler:
                            self[ typ ] = handler
                            if hasattr( handler, 'registerEquivalent' ):
                                handler.registerEquivalent( typ, base )
                            return handler
                print(self.keys())
                raise TypeError(
                    """No array-type handler for type %s.%s (value: %s) registered"""%(
                        typ.__module__, typ.__name__, repr(value)[:50]
                    )
                )
            return handler
        
        def handler_by_plugin_name( self, name ):
            plugin = plugins.FormatHandler.by_name( name )
            if plugin:
                try:
                    return plugin.load()
                except ImportError:
                    return None
            else:
                raise RuntimeError( 'No handler of name %s found'%(name,))
        
        def get_output_handler( self ):
            """Fast-path lookup for output handler object"""
            if self.output_handler is None:
                if self.preferredOutput is not None:
                    self.output_handler = self.handler_by_plugin_name( self.preferredOutput )
                if not self.output_handler:
                    for preferred in self.GENERIC_OUTPUT_PREFERENCES:
                        self.output_handler = self.handler_by_plugin_name( preferred )
                        if self.output_handler:
                            break
                if not self.output_handler:
                    raise RuntimeError(
                        """Unable to find any output handler at all (not even ctypes/numpy ones!)"""
                    )
            return self.output_handler
        
        def register( self, handler, types=None ):
            """Register this class as handler for given set of types"""
            if not isinstance( types, (list,tuple)):
                types = [ types ]
            for type in types:
                self[ type ] = handler
            if handler.isOutput:
                self.all_output_handlers.append( handler )
            
        def registerReturn( self, handler ):
            """Register this handler as the default return-type handler"""
            if isinstance( handler, (str,unicode)):
                self.preferredOutput = handler 
                self.output_handler = None
            else:
                self.preferredOutput = None 
                self.output_handler = handler 
    
    GLOBAL_REGISTRY = HandlerRegistry( plugins.FormatHandler.match)
    formathandler.FormatHandler.TYPE_REGISTRY = GLOBAL_REGISTRY
    
    class ArrayDatatype( object ):
        """Mix-in for array datatype classes
        
        The ArrayDatatype marker essentially is used to mark a particular argument
        as having an "array" type, which means that it is eligible for handling 
        via the arrays sub-package and its registered handlers.
        """
        typeConstant = None
        handler = GLOBAL_REGISTRY
        getHandler = GLOBAL_REGISTRY.__call__
        returnHandler = GLOBAL_REGISTRY.get_output_handler
        isAccelerated = False
        @classmethod
        def getRegistry( cls ):
            """Get our handler registry"""
            return cls.handler 
        def from_param( cls, value, typeConstant=None ):
            """Given a value in a known data-pointer type, convert to a ctypes pointer"""
            return cls.getHandler(value).from_param( value, cls.typeConstant )
        from_param = classmethod( logs.logOnFail( from_param, _log ) )
        def dataPointer( cls, value ):
            """Given a value in a known data-pointer type, return long for pointer"""
            try:
                return cls.getHandler(value).dataPointer( value )
            except Exception:
                _log.warning(
                    """Failure in dataPointer for %s instance %s""", type(value), value,
                )
                raise
        dataPointer = classmethod( logs.logOnFail( dataPointer, _log ) )
        def voidDataPointer( cls, value ):
            """Given value in a known data-pointer type, return void_p for pointer"""
            pointer = cls.dataPointer( value )
            try:
                return ctypes.c_void_p(pointer)
            except TypeError:
                return pointer
        voidDataPointer = classmethod( logs.logOnFail( voidDataPointer, _log ) )
        def typedPointer( cls, value ):
            """Return a pointer-to-base-type pointer for given value"""
            return ctypes.cast( cls.dataPointer(value), ctypes.POINTER( cls.baseType ))
        typedPointer = classmethod( typedPointer )
        def asArray( cls, value, typeCode=None ):
            """Given a value, convert to preferred array representation"""
            return cls.getHandler(value).asArray( value, typeCode or cls.typeConstant )
        asArray = classmethod( logs.logOnFail( asArray, _log ) )
        def arrayToGLType( cls, value ):
            """Given a data-value, guess the OpenGL type of the corresponding pointer
            
            Note: this is not currently used in PyOpenGL and may be removed 
            eventually.
            """
            return cls.getHandler(value).arrayToGLType( value )
        arrayToGLType = classmethod( logs.logOnFail( arrayToGLType, _log ) )
        def arraySize( cls, value, typeCode = None ):
            """Given a data-value, calculate dimensions for the array (number-of-units)"""
            return cls.getHandler(value).arraySize( value, typeCode or cls.typeConstant )
        arraySize = classmethod( logs.logOnFail( arraySize, _log ) )
        def unitSize( cls, value, typeCode=None ):
            """Determine unit size of an array (if possible)
            
            Uses our local type if defined, otherwise asks the handler to guess...
            """
            return cls.getHandler(value).unitSize( value, typeCode or cls.typeConstant )
        unitSize = classmethod( logs.logOnFail( unitSize, _log ) )
        def zeros( cls, dims, typeCode=None ):
            """Allocate a return array of the given dimensions filled with zeros"""
            return cls.returnHandler().zeros( dims, typeCode or cls.typeConstant )
        zeros = classmethod( logs.logOnFail( zeros, _log ) )
        def dimensions( cls, value ):
            """Given a data-value, get the dimensions (assumes full structure info)"""
            return cls.getHandler(value).dimensions( value )
        dimensions = classmethod( logs.logOnFail( dimensions, _log ) )
        
        def arrayByteCount( cls, value ):
            """Given a data-value, try to determine number of bytes it's final form occupies
            
            For most data-types this is arraySize() * atomic-unit-size
            """
            return cls.getHandler(value).arrayByteCount( value )
        arrayByteCount = classmethod( logs.logOnFail( arrayByteCount, _log ) )
            

    # the final array data-type classes...
    class GLclampdArray( ArrayDatatype, ctypes.POINTER(_types.GLclampd )):
        """Array datatype for GLclampd types"""
        baseType = _types.GLclampd
        typeConstant = _types.GL_DOUBLE

    class GLclampfArray( ArrayDatatype, ctypes.POINTER(_types.GLclampf )):
        """Array datatype for GLclampf types"""
        baseType = _types.GLclampf
        typeConstant = _types.GL_FLOAT

    class GLfloat16Array(ArrayDatatype, ctypes.POINTER(_types.GLushort)):
        """Array datatype for float16 as GLushort types"""
        baseType = _types.GLushort
        typeConstant = _types.GL_HALF_FLOAT

    class GLfloatArray( ArrayDatatype, ctypes.POINTER(_types.GLfloat )):
        """Array datatype for GLfloat types"""
        baseType = _types.GLfloat
        typeConstant = _types.GL_FLOAT

    class GLdoubleArray( ArrayDatatype, ctypes.POINTER(_types.GLdouble )):
        """Array datatype for GLdouble types"""
        baseType = _types.GLdouble
        typeConstant = _types.GL_DOUBLE

    class GLbyteArray( ArrayDatatype, ctypes.POINTER(_types.GLbyte )):
        """Array datatype for GLbyte types"""
        baseType = _types.GLbyte
        typeConstant = _types.GL_BYTE

    class GLcharArray( ArrayDatatype, ctypes.c_char_p):
        """Array datatype for ARB extension pointers-to-arrays"""
        baseType = _types.GLchar
        typeConstant = _types.GL_BYTE
    GLcharARBArray = GLcharArray

    class GLshortArray( ArrayDatatype, ctypes.POINTER(_types.GLshort )):
        """Array datatype for GLshort types"""
        baseType = _types.GLshort
        typeConstant = _types.GL_SHORT

    class GLintArray( ArrayDatatype, ctypes.POINTER(_types.GLint )):
        """Array datatype for GLint types"""
        baseType = _types.GLint
        typeConstant = _types.GL_INT

    class GLubyteArray( ArrayDatatype, ctypes.POINTER(_types.GLubyte )):
        """Array datatype for GLubyte types"""
        baseType = _types.GLubyte
        typeConstant = _types.GL_UNSIGNED_BYTE
    GLbooleanArray = GLubyteArray

    class GLushortArray( ArrayDatatype, ctypes.POINTER(_types.GLushort )):
        """Array datatype for GLushort types"""
        baseType = _types.GLushort
        typeConstant = _types.GL_UNSIGNED_SHORT

    class GLuintArray( ArrayDatatype, ctypes.POINTER(_types.GLuint )):
        """Array datatype for GLuint types"""
        baseType = _types.GLuint
        typeConstant = _types.GL_UNSIGNED_INT
    
    class GLint64Array( ArrayDatatype, ctypes.POINTER(_types.GLint64 )):
        """Array datatype for GLuint types"""
        baseType = _types.GLint64
        typeConstant = None # TODO: find out what this should be!
    
    class GLuint64Array( ArrayDatatype, ctypes.POINTER(_types.GLuint64 )):
        """Array datatype for GLuint types"""
        baseType = _types.GLuint64
        typeConstant = _types.GL_UNSIGNED_INT64

    class GLenumArray( ArrayDatatype, ctypes.POINTER(_types.GLenum )):
        """Array datatype for GLenum types"""
        baseType = _types.GLenum
        typeConstant = _types.GL_UNSIGNED_INT
    class GLsizeiArray( ArrayDatatype, ctypes.POINTER(_types.GLsizei )):
        """Array datatype for GLsizei types"""
        baseType = _types.GLsizei
        typeConstant = _types.GL_INT
    class GLvoidpArray( ArrayDatatype, ctypes.POINTER(_types.GLvoid )):
        """Array datatype for GLenum types"""
        baseType = _types.GLvoidp
        typeConstant = _types.GL_VOID_P
    
    class GLfixedArray(ArrayDatatype, ctypes.POINTER(_types.GLfixed)):
        baseType = _types.GLfixed 
        typeConstant = _types.GL_FIXED
    
else:
    # Cython-coded array handler
    _log.info( 'Using accelerated ArrayDatatype' )
    ArrayDatatype = ADT( None, None )
    GLclampdArray = ADT( GL_1_1.GL_DOUBLE, _types.GLclampd )
    GLclampfArray = ADT( GL_1_1.GL_FLOAT, _types.GLclampf )
    GLdoubleArray = ADT( GL_1_1.GL_DOUBLE, _types.GLdouble )
    GLfloat16Array = ADT( GL_1_1.GL_HALF_FLOAT, _types.GLushort )
    GLfloatArray = ADT( GL_1_1.GL_FLOAT, _types.GLfloat )
    GLbyteArray = ADT( GL_1_1.GL_BYTE, _types.GLbyte )
    GLcharArray = GLcharARBArray = ADT( GL_1_1.GL_BYTE, _types.GLchar )
    GLshortArray = ADT( GL_1_1.GL_SHORT, _types.GLshort )
    GLintArray = ADT( GL_1_1.GL_INT, _types.GLint )
    GLubyteArray = GLbooleanArray = ADT( GL_1_1.GL_UNSIGNED_BYTE, _types.GLubyte )
    GLushortArray = ADT( GL_1_1.GL_UNSIGNED_SHORT, _types.GLushort )
    GLuintArray = ADT( GL_1_1.GL_UNSIGNED_INT, _types.GLuint )
    GLint64Array = ADT( None, _types.GLint64 )
    GLuint64Array = ADT( GL_1_1.GL_UNSIGNED_INT64, _types.GLuint64 )
    GLenumArray = ADT( GL_1_1.GL_UNSIGNED_INT, _types.GLenum )
    GLsizeiArray = ADT( GL_1_1.GL_INT, _types.GLsizei )
    GLvoidpArray = ADT( _types.GL_VOID_P, _types.GLvoidp )
    GLfixedArray = ADT( _types.GL_FIXED, _types.GLfixed )

EGLAttribArray = GLintArray


GL_CONSTANT_TO_ARRAY_TYPE = {
    GL_1_1.GL_HALF_FLOAT : GLfloat16Array,
    GL_1_1.GL_DOUBLE : GLclampdArray,
    GL_1_1.GL_FLOAT : GLclampfArray,
    GL_1_1.GL_FLOAT : GLfloatArray,
    GL_1_1.GL_DOUBLE : GLdoubleArray,
    GL_1_1.GL_BYTE : GLbyteArray,
    GL_1_1.GL_SHORT : GLshortArray,
    GL_1_1.GL_INT : GLintArray,
    GL_1_1.GL_UNSIGNED_BYTE : GLubyteArray,
    GL_1_1.GL_UNSIGNED_SHORT : GLushortArray,
    GL_1_1.GL_UNSIGNED_INT : GLuintArray,
    _types.GL_FIXED: GLfixedArray, 
    #GL_1_1.GL_UNSIGNED_INT : GLenumArray,
}



'''OpenGL extension MESA.shader_integer_functions

This module customises the behaviour of the 
OpenGL.raw.GL.MESA.shader_integer_functions to provide a more 
Python-friendly API

Overview (from the spec)
	
	GL_ARB_gpu_shader5 extends GLSL in a number of useful ways.  Much of this
	added functionality requires significant hardware support.  There are many
	aspects, however, that can be easily implmented on any GPU with "real"
	integer support (as opposed to simulating integers using floating point
	calculations).
	
	This extension provides a set of new features to the OpenGL Shading
	Language to support capabilities of these GPUs, extending the
	capabilities of version 1.30 of the OpenGL Shading Language and version
	3.00 of the OpenGL ES Shading Language.  Shaders using the new
	functionality provided by this extension should enable this
	functionality via the construct
	
	  #extension GL_MESA_shader_integer_functions : require   (or enable)
	
	This extension provides a variety of new features for all shader types,
	including:
	
	  * support for implicitly converting signed integer types to unsigned
	    types, as well as more general implicit conversion and function
	    overloading infrastructure to support new data types introduced by
	    other extensions;
	
	  * new built-in functions supporting:
	
	    * splitting a floating-point number into a significand and exponent
	      (frexp), or building a floating-point number from a significand and
	      exponent (ldexp);
	
	    * integer bitfield manipulation, including functions to find the
	      position of the most or least significant set bit, count the number
	      of one bits, and bitfield insertion, extraction, and reversal;
	
	    * extended integer precision math, including add with carry, subtract
	      with borrow, and extenended multiplication;
	
	The resulting extension is a strict subset of GL_ARB_gpu_shader5.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/shader_integer_functions.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESA.shader_integer_functions import *
from OpenGL.raw.GL.MESA.shader_integer_functions import _EXTENSION_NAME

def glInitShaderIntegerFunctionsMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension DMP.shader_binary

This module customises the behaviour of the 
OpenGL.raw.GLES2.DMP.shader_binary to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables loading precompiled binary shaders compatible with
	chips designed by Digital Media Professionals Inc.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/DMP/shader_binary.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.DMP.shader_binary import *
from OpenGL.raw.GLES2.DMP.shader_binary import _EXTENSION_NAME

def glInitShaderBinaryDMP():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.multitexture

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.multitexture to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/multitexture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.multitexture import *
from OpenGL.raw.GL.ARB.multitexture import _EXTENSION_NAME

def glInitMultitextureARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glMultiTexCoord1dvARB=wrapper.wrapper(glMultiTexCoord1dvARB).setInputArraySize(
    'v', 1
)
glMultiTexCoord1fvARB=wrapper.wrapper(glMultiTexCoord1fvARB).setInputArraySize(
    'v', 1
)
glMultiTexCoord1ivARB=wrapper.wrapper(glMultiTexCoord1ivARB).setInputArraySize(
    'v', 1
)
glMultiTexCoord1svARB=wrapper.wrapper(glMultiTexCoord1svARB).setInputArraySize(
    'v', 1
)
glMultiTexCoord2dvARB=wrapper.wrapper(glMultiTexCoord2dvARB).setInputArraySize(
    'v', 2
)
glMultiTexCoord2fvARB=wrapper.wrapper(glMultiTexCoord2fvARB).setInputArraySize(
    'v', 2
)
glMultiTexCoord2ivARB=wrapper.wrapper(glMultiTexCoord2ivARB).setInputArraySize(
    'v', 2
)
glMultiTexCoord2svARB=wrapper.wrapper(glMultiTexCoord2svARB).setInputArraySize(
    'v', 2
)
glMultiTexCoord3dvARB=wrapper.wrapper(glMultiTexCoord3dvARB).setInputArraySize(
    'v', 3
)
glMultiTexCoord3fvARB=wrapper.wrapper(glMultiTexCoord3fvARB).setInputArraySize(
    'v', 3
)
glMultiTexCoord3ivARB=wrapper.wrapper(glMultiTexCoord3ivARB).setInputArraySize(
    'v', 3
)
glMultiTexCoord3svARB=wrapper.wrapper(glMultiTexCoord3svARB).setInputArraySize(
    'v', 3
)
glMultiTexCoord4dvARB=wrapper.wrapper(glMultiTexCoord4dvARB).setInputArraySize(
    'v', 4
)
glMultiTexCoord4fvARB=wrapper.wrapper(glMultiTexCoord4fvARB).setInputArraySize(
    'v', 4
)
glMultiTexCoord4ivARB=wrapper.wrapper(glMultiTexCoord4ivARB).setInputArraySize(
    'v', 4
)
glMultiTexCoord4svARB=wrapper.wrapper(glMultiTexCoord4svARB).setInputArraySize(
    'v', 4
)
### END AUTOGENERATED SECTION



'''OpenGL extension AMD.framebuffer_multisample_advanced

This module customises the behaviour of the 
OpenGL.raw.GLES2.AMD.framebuffer_multisample_advanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends ARB_framebuffer_object by allowing compromises
	between image quality and memory footprint of multisample
	antialiasing.
	
	ARB_framebuffer_object introduced RenderbufferStorageMultisample
	as a method of defining the parameters for a multisample render
	buffer. This function takes a <samples> parameter that has strict
	requirements on behavior such that no compromises in the final image
	quality are allowed. Additionally, ARB_framebuffer_object requires
	that all framebuffer attachments have the same number of samples.
	
	This extension extends ARB_framebuffer_object by providing a new
	function, RenderbufferStorageMultisampleAdvancedAMD, that
	distinguishes between samples and storage samples for color
	renderbuffers where the number of storage samples can be less than
	the number of samples. This extension also allows non-matching sample
	counts between color and depth/stencil renderbuffers.
	
	This extension does not require any specific combination of sample
	counts to be supported.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/framebuffer_multisample_advanced.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.AMD.framebuffer_multisample_advanced import *
from OpenGL.raw.GLES2.AMD.framebuffer_multisample_advanced import _EXTENSION_NAME

def glInitFramebufferMultisampleAdvancedAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension IMG.texture_filter_cubic

This module customises the behaviour of the 
OpenGL.raw.GLES2.IMG.texture_filter_cubic to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL ES provides two sampling methods available; nearest neighbor or
	linear filtering, with optional MIP Map sampling modes added to move between
	differently sized textures when downsampling.
	
	This extension adds an additional, high quality cubic filtering mode, using
	a Catmull-Rom bicubic filter. Performing this kind of filtering can be done
	in a shader by using 16 samples, but this can be inefficient. The cubic
	filter mode exposes an optimized high quality texture sampling using fixed
	functionality.
	
	This extension affects the way textures are sampled, by modifying the way
	texels within the same MIP-Map level are sampled and resolved. It does not
	affect MIP-Map filtering, which is still limited to linear or nearest.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/texture_filter_cubic.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.IMG.texture_filter_cubic import *
from OpenGL.raw.GLES2.IMG.texture_filter_cubic import _EXTENSION_NAME

def glInitTextureFilterCubicIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_MESA_tile_raster_order'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_MESA_tile_raster_order',error_checker=_errors._error_checker)
GL_TILE_RASTER_ORDER_FIXED_MESA=_C('GL_TILE_RASTER_ORDER_FIXED_MESA',0x8BB8)
GL_TILE_RASTER_ORDER_INCREASING_X_MESA=_C('GL_TILE_RASTER_ORDER_INCREASING_X_MESA',0x8BB9)
GL_TILE_RASTER_ORDER_INCREASING_Y_MESA=_C('GL_TILE_RASTER_ORDER_INCREASING_Y_MESA',0x8BBA)




'''OpenGL extension OES.depth_texture

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.depth_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a new texture format that stores depth values in 
	the texture.  Depth texture images are widely used for shadow casting but 
	can also  be used for other effects such as image based rendering, displacement 
	mapping etc.  

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/depth_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.depth_texture import *
from OpenGL.raw.GLES2.OES.depth_texture import _EXTENSION_NAME

def glInitDepthTextureOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_window_rectangles'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_window_rectangles',error_checker=_errors._error_checker)
GL_EXCLUSIVE_EXT=_C('GL_EXCLUSIVE_EXT',0x8F11)
GL_INCLUSIVE_EXT=_C('GL_INCLUSIVE_EXT',0x8F10)
GL_MAX_WINDOW_RECTANGLES_EXT=_C('GL_MAX_WINDOW_RECTANGLES_EXT',0x8F14)
GL_NUM_WINDOW_RECTANGLES_EXT=_C('GL_NUM_WINDOW_RECTANGLES_EXT',0x8F15)
GL_WINDOW_RECTANGLE_EXT=_C('GL_WINDOW_RECTANGLE_EXT',0x8F12)
GL_WINDOW_RECTANGLE_MODE_EXT=_C('GL_WINDOW_RECTANGLE_MODE_EXT',0x8F13)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLintArray)
def glWindowRectanglesEXT(mode,count,box):pass



'''OpenGL extension SGIX.ycrcba

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.ycrcba to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a method for OpenGL to read and store
	images that are defined in standard YCRCB and YCRCBA video color
	spaces.  As with the CYMK extension, conversion to RGBA takes place
	immediately following the unpack pixel store, and preceding the
	pack pixel store operations, and is only available on transfers to
	and from memory.  The algorithms that convert between YCRCB and
	RGBA are "black-box" in nature, and left undefined by the
	extension, however it is recommended that conversion comply with
	the CCIR standard. This extension specifies the format of a pixel
	rectangle independent of component subsampling. Component
	subsampling is specified using a separate extension.
	
	Two new formats are added, YCRCB_SGIX and YCRCBA_SGIX.
	
	YCRCB_SGIX is defined as a 3 component format representing the Cb,
	Y, and Cr values per pixel. YCRCBA_SGIX is defined as a 4 component
	format representing Cb, Y, Cr, and A values per pixel.
	
	As with the CMYK extension, this extension doesn't preclude the
	possibility of other higher quality conversion methods.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/ycrcba.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.ycrcba import *
from OpenGL.raw.GL.SGIX.ycrcba import _EXTENSION_NAME

def glInitYcrcbaSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.packed_depth_stencil

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.packed_depth_stencil to provide a more 
Python-friendly API

Overview (from the spec)
	
	Many OpenGL implementations have chosen to interleave the depth and stencil
	buffers into one buffer, often with 24 bits of depth precision and 8 bits of
	stencil data.  32 bits is more than is needed for the depth buffer much of 
	the time; a 24-bit depth buffer, on the other hand, requires that reads and
	writes of depth data be unaligned with respect to power-of-two boundaries.
	On the other hand, 8 bits of stencil data is more than sufficient for most
	applications, so it is only natural to pack the two buffers into a single 
	buffer with both depth and stencil data.  OpenGL never provides direct 
	access to the buffers, so the OpenGL implementation can provide an interface
	to applications where it appears the one merged buffer is composed of two
	logical buffers.
	
	One disadvantage of this scheme is that OpenGL lacks any means by which this
	packed data can be handled efficiently.  For example, when an application
	reads from the 24-bit depth buffer, using the type GL_UNSIGNED_SHORT will 
	lose 8 bits of data, while GL_UNSIGNED_INT has 8 too many.  Both require 
	expensive format conversion operations.  A 24-bit format would be no more 
	suitable, because it would also suffer from the unaligned memory accesses 
	that made the standalone 24-bit depth buffer an unattractive proposition in
	the first place.
	
	If OES_depth_texture is supported, a new data format, GL_DEPTH_STENCIL_OES,
	as well as a packed data type, UNSIGNED_INT_24_8_OES, together can be used
	with glTex[Sub]Image2D.  This provides an efficient way to supply data for a
	24-bit depth texture.  When a texture with DEPTH_STENCIL_OES data is bound
	for texturing, only the depth component is accessible through the texture
	fetcher.  
	
	This extension also provides a new sized internal format,
	DEPTH24_STENCIL8_OES, which can be used for renderbuffer storage.  When a
	renderbuffer or texture image with a DEPTH_STENCIL_OES base internal format
	is attached to both the depth and stencil attachment points of a framebuffer
	object, then it becomes both the depth and stencil buffers of the
	framebuffer.  This fits nicely with hardware that interleaves both depth and
	stencil data into a single buffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/packed_depth_stencil.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.packed_depth_stencil import *
from OpenGL.raw.GLES2.OES.packed_depth_stencil import _EXTENSION_NAME

def glInitPackedDepthStencilOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""8-bit string definitions for Python 2/3 compatibility

Defines the following which allow for dealing with Python 3 breakages:

    STR_IS_BYTES
    STR_IS_UNICODE
    
        Easily checked booleans for type identities
    
    _NULL_8_BYTE
    
        An 8-bit byte with NULL (0) value 
    
    as_8_bit( x, encoding='utf-8')
    
        Returns the value as the 8-bit version
    
    unicode -- always pointing to the unicode type 
    bytes -- always pointing to the 8-bit bytes type
"""
import sys

STR_IS_BYTES = True

if sys.version_info[:2] < (2,6):
    # no bytes, traditional setup...
    bytes = str 
else:
    bytes = bytes
try:
    long = long
except NameError as err:
    long = int
if sys.version_info[:2] < (3,0):
    # traditional setup, with bytes defined...
    unicode = unicode
    _NULL_8_BYTE = '\000'
    def as_8_bit( x, encoding='utf-8' ):
        if isinstance( x, unicode ):
            return x.encode( encoding )
        return bytes( x )
    integer_types = int,long
    def as_str( x, encoding='utf-8'):
        """Produce a native string (i.e. different on python 2 and 3)"""
        if isinstance(x,bytes):
            return x
        elif isinstance(x,unicode):
            return x.encode(encoding)
        else:
            return str(x)
else:
    # new setup, str is now unicode...
    STR_IS_BYTES = False
    _NULL_8_BYTE = bytes( '\000','latin1' )
    def as_8_bit( x, encoding='utf-8' ):
        if isinstance( x,unicode ):
            return x.encode(encoding)
        elif isinstance( x, bytes ):
            # Note: this can create an 8-bit string that is *not* in encoding,
            # but that is potentially exactly what we wanted, as these can 
            # be arbitrary byte-streams being passed to C functions
            return x
        return str(x).encode( encoding )
    unicode = str
    integer_types = int,
    def as_str( x, encoding='utf-8'):
        """Produce a native string (i.e. different on python 2 and 3)"""
        if isinstance(x,unicode):
            return x
        elif isinstance(x,bytes):
            return x.decode(encoding)
        else:
            return str(x)

STR_IS_UNICODE = not STR_IS_BYTES
if hasattr( sys, 'maxsize' ):
    maxsize = sys.maxsize 
else:
    maxsize = sys.maxint

def as_unicode(x,encoding='utf-8'):
    """Ensure is a unicode object given default encoding"""
    if isinstance(x,unicode):
        return x
    elif isinstance(x,bytes):
        try:
            return x.decode(encoding)
        except UnicodeDecodeError as err:
            return x.decode('latin-1')
    else:
        return unicode(x)




'''OpenGL extension OES.EGL_image_external

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.EGL_image_external to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a mechanism for creating EGLImage texture targets
	from EGLImages.  This extension defines a new texture target,
	TEXTURE_EXTERNAL_OES.  This texture target can only be specified using an
	EGLImage.  There is no support for most of the functions that manipulate
	other texture targets (e.g. you cannot use gl*Tex*Image*() functions with
	TEXTURE_EXTERNAL_OES).  Also, TEXTURE_EXTERNAL_OES targets never have more
	than a single LOD level.  Because of these restrictions, it is possible to
	bind EGLImages which have internal formats not otherwise supported by
	OpenGL ES.  For example some implementations may allow EGLImages with
	planar or interleaved YUV data to be GLES texture target siblings.  It is
	up to the implementation exactly what formats are accepted.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/EGL_image_external.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.EGL_image_external import *
from OpenGL.raw.GLES1.OES.EGL_image_external import _EXTENSION_NAME

def glInitEglImageExternalOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.draw_buffers

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.draw_buffers to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension extends OpenGL ES 2.0 to allow multiple output
	colors, and provides a mechanism for directing those outputs to
	multiple color buffers.
	
	This extension serves a similar purpose to ARB_draw_buffers except
	that this is for OpenGL ES 2.0.
	
	When OpenGL ES 3.0 is present, this extension relaxes the order
	restriction on color attachments to draw framebuffer objects.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/draw_buffers.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.draw_buffers import *
from OpenGL.raw.GLES2.NV.draw_buffers import _EXTENSION_NAME

def glInitDrawBuffersNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawBuffersNV.bufs size not checked against n
glDrawBuffersNV=wrapper.wrapper(glDrawBuffersNV).setInputArraySize(
    'bufs', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension APPLE.copy_texture_levels

This module customises the behaviour of the 
OpenGL.raw.GLES2.APPLE.copy_texture_levels to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides an efficient path for copying a contiguous subset of mipmap 
	levels from one texture to the matching subset of mipmap levels of another texture, 
	where matches are determined by the equality of a level's dimensions.
	
	This extension is dependent on the existence of the extension EXT_texture_storage.
	Immutable textures are used to guarantee that storage is allocated up front for the
	source and destination textures and that the internal formats of those textures are 
	sized the same.
	
	An efficient copy can be achieved by implementations because the internal storage 
	requirements are the same between textures and will remain unchanged when moving data. 
	It is expected that in all cases, moving levels from one texture to another is a 
	simple copy operation without any necessary conversion. This extension can be used as
	an alternative to TEXTURE_BASE_LEVEL. In some implementations, changing the value of
	TEXTURE_BASE_LEVEL can incur a costly re-allocation at runtime.
	
	Texture streaming is an expected use case for this extension. For example, a developer
	may want to stream in a larger base level for a given texture from a storage device. 
	To achieve this, a copy of the current mipmap levels are made into a destination 
	whose storage was specified to accommodate the source levels and the larger base 
	level. The efficiency of the copy without conversion allows for the smaller mipmap 
	levels to be in place while the larger base level is being read from the storage 
	device and uploaded.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/copy_texture_levels.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.APPLE.copy_texture_levels import *
from OpenGL.raw.GLES2.APPLE.copy_texture_levels import _EXTENSION_NAME

def glInitCopyTextureLevelsAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGI.color_table

This module customises the behaviour of the 
OpenGL.raw.GL.SGI.color_table to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a new RGBA-format color lookup mechanism.  It does
	not replace the color lookups defined by the GL Specification, but rather
	provides additional lookup capabilities with different operation.  The key
	difference is that the new lookup tables are treated as 1-dimensional images
	with internal formats, like texture images and convolution filter images.
	From this follows the fact that the new tables can operate on a subset of
	the components of passing pixel groups.  For example, a table with internal
	format ALPHA modifies only the A component of each pixel group, leaving the
	R, G, and B components unmodified.
	
	If EXT_copy_texture is implemented, this extension also defines methods to
	initialize the color lookup tables from the framebuffer, in addition to the
	standard memory source mechanisms.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGI/color_table.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGI.color_table import *
from OpenGL.raw.GL.SGI.color_table import _EXTENSION_NAME

def glInitColorTableSGI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glColorTableSGI.table size not checked against 'format,type,width'
glColorTableSGI=wrapper.wrapper(glColorTableSGI).setInputArraySize(
    'table', None
)
# INPUT glColorTableParameterfvSGI.params size not checked against 'pname'
glColorTableParameterfvSGI=wrapper.wrapper(glColorTableParameterfvSGI).setInputArraySize(
    'params', None
)
# INPUT glColorTableParameterivSGI.params size not checked against 'pname'
glColorTableParameterivSGI=wrapper.wrapper(glColorTableParameterivSGI).setInputArraySize(
    'params', None
)
# OUTPUT glGetColorTableSGI.table COMPSIZE(target, format, type) 
glGetColorTableParameterfvSGI=wrapper.wrapper(glGetColorTableParameterfvSGI).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetColorTableParameterivSGI=wrapper.wrapper(glGetColorTableParameterivSGI).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension NV.gpu_multicast

This module customises the behaviour of the 
OpenGL.raw.GL.NV.gpu_multicast to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables novel multi-GPU rendering techniques by providing application control
	over a group of linked GPUs with identical hardware configuration.
	
	Multi-GPU rendering techniques fall into two categories: implicit and explicit.  Existing
	explicit approaches like WGL_NV_gpu_affinity have two main drawbacks: CPU overhead and
	application complexity.  An application must manage one context per GPU and multi-pump the API
	stream.  Implicit multi-GPU rendering techniques avoid these issues by broadcasting rendering
	from one context to multiple GPUs.  Common implicit approaches include alternate-frame
	rendering (AFR), split-frame rendering (SFR) and multi-GPU anti-aliasing.  They each have
	drawbacks.  AFR scales nicely but interacts poorly with inter-frame dependencies.  SFR can
	improve latency but has challenges with offscreen rendering and scaling of vertex processing.
	With multi-GPU anti-aliasing, each GPU renders the same content with alternate sample
	positions and the driver blends the result to improve quality.  This also has issues with
	offscreen rendering and can conflict with other anti-aliasing techniques.
	
	These issues with implicit multi-GPU rendering all have the same root cause: the driver lacks
	adequate knowledge to accelerate every application.  To resolve this, NV_gpu_multicast
	provides fine-grained, explicit application control over multiple GPUs with a single context.
	
	Key points:
	
	- One context controls multiple GPUs.  Every GPU in the linked group can access every object.
	
	- Rendering is broadcast.  Each draw is repeated across all GPUs in the linked group.
	
	- Each GPU gets its own instance of all framebuffers, allowing individualized output for each
	  GPU.  Input data can be customized for each GPU using buffers created with the storage flag,
	  PER_GPU_STORAGE_BIT_NV and a new API, MulticastBufferSubDataNV. 
	
	- New interfaces provide mechanisms to transfer textures and buffers from one GPU to another.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/gpu_multicast.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.gpu_multicast import *
from OpenGL.raw.GL.NV.gpu_multicast import _EXTENSION_NAME

def glInitGpuMulticastNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.texture_npot

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.texture_npot to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for the REPEAT and MIRRORED_REPEAT
	texture wrap modes and the minification filters supported for
	non-power of two 2D textures, cubemaps and for 3D textures, if
	the OES_texture_3D extension is supported.
	
	Section 3.8.2 of the OpenGL ES 2.0 specification describes
	rules for sampling from an incomplete texture. There were specific
	rules added for non-power of two textures i.e. if the texture wrap
	mode is not CLAMP_TO_EDGE or minification filter is not NEAREST or
	LINEAR and the texture is a non-power-of-two texture, then sampling
	the texture will return (0, 0, 0, 1).
	
	These rules are no longer applied by an implementation that supports
	this extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_npot.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.texture_npot import *
from OpenGL.raw.GLES1.OES.texture_npot import _EXTENSION_NAME

def glInitTextureNpotOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.DISABLED import _types as _cs
# End users want this...
from OpenGL.raw.DISABLED._types import *
from OpenGL.raw.DISABLED import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'DISABLED_NVX_cross_process_interop'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.DISABLED,'DISABLED_NVX_cross_process_interop',error_checker=_errors._error_checker)
GL_EXTERNAL_STORAGE_BIT_NVX=_C('GL_EXTERNAL_STORAGE_BIT_NVX',0x2000)




'''OpenGL extension NV.alpha_to_coverage_dither_control

This module customises the behaviour of the 
OpenGL.raw.GL.NV.alpha_to_coverage_dither_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	NV_alpha_to_coverage_dither_control_triangles provides a new mechanism
	to control whether dithering is applied when the existing alpha to coverage
	functionality is used.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/alpha_to_coverage_dither_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.alpha_to_coverage_dither_control import *
from OpenGL.raw.GL.NV.alpha_to_coverage_dither_control import _EXTENSION_NAME

def glInitAlphaToCoverageDitherControlNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.scalebias_hint

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.scalebias_hint to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds an precision hint for scaling and biasing
	arithmetic. For large scale factors, hinting GL_NICEST produces the
	most accurate results. The hint only applies to Octane2 VPro
	graphics systems, to work around a hardware issue.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/scalebias_hint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.scalebias_hint import *
from OpenGL.raw.GL.SGIX.scalebias_hint import _EXTENSION_NAME

def glInitScalebiasHintSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


from OpenGL.platform import PLATFORM as _p
from OpenGL.error import _ErrorChecker
if _ErrorChecker:
    _error_checker = _ErrorChecker( _p, getattr(_p.GLES3,'glGetError',None) )
else:
    _error_checker = None




'''OpenGL extension EXT.gpu_shader5

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.gpu_shader5 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a set of new features to the OpenGL ES Shading
	Language and related APIs to support capabilities of new GPUs, extending
	the capabilities of version 3.10 of the OpenGL ES Shading Language.
	Shaders using the new functionality provided by this extension should
	enable this functionality via the construct
	
	  #extension GL_EXT_gpu_shader5 : require     (or enable)
	
	This extension provides a variety of new features for all shader types,
	including:
	
	  * support for indexing into arrays of opaque types (samplers,
	    and atomic counters) using dynamically uniform integer expressions;
	
	  * support for indexing into arrays of images and shader storage blocks
	    using only constant integral expressions;
	
	  * extending the uniform block capability to allow shaders to index
	    into an array of uniform blocks;
	
	  * a "precise" qualifier allowing computations to be carried out exactly
	    as specified in the shader source to avoid optimization-induced
	    invariance issues (which might cause cracking in tessellation);
	
	  * new built-in functions supporting:
	
	    * fused floating-point multiply-add operations;
	
	  * extending the textureGather() built-in functions provided by
	    OpenGL ES Shading Language 3.10:
	
	    * allowing shaders to use arbitrary offsets computed at run-time to
	      select a 2x2 footprint to gather from; and
	    * allowing shaders to use separate independent offsets for each of
	      the four texels returned, instead of requiring a fixed 2x2
	      footprint.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/gpu_shader5.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.gpu_shader5 import *
from OpenGL.raw.GLES2.EXT.gpu_shader5 import _EXTENSION_NAME

def glInitGpuShader5EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NVX.gpu_multicast2

This module customises the behaviour of the 
OpenGL.raw.GL.NVX.gpu_multicast2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional mechanisms that influence multicast rendering which is
	simultaneous rendering to multiple GPUs.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NVX/gpu_multicast2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NVX.gpu_multicast2 import *
from OpenGL.raw.GL.NVX.gpu_multicast2 import _EXTENSION_NAME

def glInitGpuMulticast2NVX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glMulticastViewportArrayvNVX.v size not checked against 'count'
glMulticastViewportArrayvNVX=wrapper.wrapper(glMulticastViewportArrayvNVX).setInputArraySize(
    'v', None
)
# INPUT glMulticastScissorArrayvNVX.v size not checked against 'count'
glMulticastScissorArrayvNVX=wrapper.wrapper(glMulticastScissorArrayvNVX).setInputArraySize(
    'v', None
)
# INPUT glAsyncCopyBufferSubDataNVX.fenceValueArray size not checked against waitSemaphoreCount
# INPUT glAsyncCopyBufferSubDataNVX.signalSemaphoreArray size not checked against signalSemaphoreCount
# INPUT glAsyncCopyBufferSubDataNVX.signalValueArray size not checked against signalSemaphoreCount
# INPUT glAsyncCopyBufferSubDataNVX.waitSemaphoreArray size not checked against waitSemaphoreCount
glAsyncCopyBufferSubDataNVX=wrapper.wrapper(glAsyncCopyBufferSubDataNVX).setInputArraySize(
    'fenceValueArray', None
).setInputArraySize(
    'signalSemaphoreArray', None
).setInputArraySize(
    'signalValueArray', None
).setInputArraySize(
    'waitSemaphoreArray', None
)
# INPUT glAsyncCopyImageSubDataNVX.signalSemaphoreArray size not checked against signalSemaphoreCount
# INPUT glAsyncCopyImageSubDataNVX.signalValueArray size not checked against signalSemaphoreCount
# INPUT glAsyncCopyImageSubDataNVX.waitSemaphoreArray size not checked against waitSemaphoreCount
# INPUT glAsyncCopyImageSubDataNVX.waitValueArray size not checked against waitSemaphoreCount
glAsyncCopyImageSubDataNVX=wrapper.wrapper(glAsyncCopyImageSubDataNVX).setInputArraySize(
    'signalSemaphoreArray', None
).setInputArraySize(
    'signalValueArray', None
).setInputArraySize(
    'waitSemaphoreArray', None
).setInputArraySize(
    'waitValueArray', None
)
### END AUTOGENERATED SECTION


"""ctypes data-pointers as a data-format mechanism
"""
REGISTRY_NAME = 'ctypespointers'
import ctypes, _ctypes
from OpenGL.raw.GL import _types 
from OpenGL.arrays import _arrayconstants as GL_1_1
from OpenGL import constant
from OpenGL.arrays import formathandler
import operator

class CtypesPointerHandler( formathandler.FormatHandler ):
    """Ctypes Pointer-type-specific data-type handler for OpenGL
    
    Because pointers do not have size information we can't use
    them for output of data, but they can be used for certain
    types of input...
    """
    @classmethod
    def from_param( cls, value, typeCode=None  ):
        return value
    @classmethod
    def dataPointer( cls, value ):
        return value.value
    HANDLED_TYPES = (ctypes._Pointer, )
    isOutput=False
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.cast( value, ctypes.c_void_p )
    def zeros( self, dims, typeCode ):
        """Return Numpy array of zeros in given size"""
        raise NotImplementedError( """Sized output doesn't yet work...""" )
    def ones( self, dims, typeCode='d' ):
        """Return numpy array of ones in given size"""
        raise NotImplementedError( """Haven't got a good ones implementation yet""" )
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                value._type_, list(ARRAY_TO_GL_TYPE_MAPPING.keys()), value,
            )
        )
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        raise NotImplementedError( """Haven't got an arraySize implementation""" )
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        return value
    def unitSize( self, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        return 1
    def dimensions( self, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        raise NotImplementedError( """Haven't got a dimensions implementation""" )


ARRAY_TO_GL_TYPE_MAPPING = {
    _types.GLdouble: GL_1_1.GL_DOUBLE,
    _types.GLfloat: GL_1_1.GL_FLOAT,
    _types.GLint: GL_1_1.GL_INT,
    _types.GLuint: GL_1_1.GL_UNSIGNED_INT,
    _types.GLshort: GL_1_1.GL_SHORT,
    _types.GLushort: GL_1_1.GL_UNSIGNED_SHORT,
        
    _types.GLchar: GL_1_1.GL_CHAR,
    _types.GLbyte: GL_1_1.GL_BYTE,
    _types.GLubyte: GL_1_1.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    GL_1_1.GL_DOUBLE: _types.GLdouble,
    GL_1_1.GL_FLOAT: _types.GLfloat,
    GL_1_1.GL_INT: _types.GLint,
    GL_1_1.GL_UNSIGNED_INT: _types.GLuint,
    GL_1_1.GL_SHORT: _types.GLshort,
    GL_1_1.GL_UNSIGNED_SHORT: _types.GLushort,
        
    GL_1_1.GL_CHAR: _types.GLchar,
    GL_1_1.GL_BYTE: _types.GLbyte,
    GL_1_1.GL_UNSIGNED_BYTE: _types.GLubyte,
    'f': _types.GLfloat,
    'd': _types.GLdouble,
    'i': _types.GLint,
    'I': _types.GLuint,
    'h': _types.GLshort,
    'H': _types.GLushort,
    'b': _types.GLbyte,
    'B': _types.GLubyte,
    's': _types.GLchar,
}



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_fence'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_fence',error_checker=_errors._error_checker)
GL_ALL_COMPLETED_NV=_C('GL_ALL_COMPLETED_NV',0x84F2)
GL_FENCE_CONDITION_NV=_C('GL_FENCE_CONDITION_NV',0x84F4)
GL_FENCE_STATUS_NV=_C('GL_FENCE_STATUS_NV',0x84F3)
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteFencesNV(n,fences):pass
@_f
@_p.types(None,_cs.GLuint)
def glFinishFenceNV(fence):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenFencesNV(n,fences):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetFenceivNV(fence,pname,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsFenceNV(fence):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glSetFenceNV(fence,condition):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glTestFenceNV(fence):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_NV_stream_metadata'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_NV_stream_metadata',error_checker=_errors._error_checker)
EGL_CONSUMER_METADATA_NV=_C('EGL_CONSUMER_METADATA_NV',0x3254)
EGL_MAX_STREAM_METADATA_BLOCKS_NV=_C('EGL_MAX_STREAM_METADATA_BLOCKS_NV',0x3250)
EGL_MAX_STREAM_METADATA_BLOCK_SIZE_NV=_C('EGL_MAX_STREAM_METADATA_BLOCK_SIZE_NV',0x3251)
EGL_MAX_STREAM_METADATA_TOTAL_SIZE_NV=_C('EGL_MAX_STREAM_METADATA_TOTAL_SIZE_NV',0x3252)
EGL_METADATA0_SIZE_NV=_C('EGL_METADATA0_SIZE_NV',0x3255)
EGL_METADATA0_TYPE_NV=_C('EGL_METADATA0_TYPE_NV',0x3259)
EGL_METADATA1_SIZE_NV=_C('EGL_METADATA1_SIZE_NV',0x3256)
EGL_METADATA1_TYPE_NV=_C('EGL_METADATA1_TYPE_NV',0x325A)
EGL_METADATA2_SIZE_NV=_C('EGL_METADATA2_SIZE_NV',0x3257)
EGL_METADATA2_TYPE_NV=_C('EGL_METADATA2_TYPE_NV',0x325B)
EGL_METADATA3_SIZE_NV=_C('EGL_METADATA3_SIZE_NV',0x3258)
EGL_METADATA3_TYPE_NV=_C('EGL_METADATA3_TYPE_NV',0x325C)
EGL_PENDING_METADATA_NV=_C('EGL_PENDING_METADATA_NV',0x3328)
EGL_PRODUCER_METADATA_NV=_C('EGL_PRODUCER_METADATA_NV',0x3253)
@_f
@_p.types(_cs.EGLBoolean,_cs.EGLDisplay,_cs.EGLint,arrays.EGLAttribArray)
def eglQueryDisplayAttribNV(dpy,attribute,value):pass
@_f
@_p.types(_cs.EGLBoolean,_cs.EGLDisplay,_cs.EGLStreamKHR,_cs.EGLenum,_cs.EGLint,_cs.EGLint,_cs.EGLint,ctypes.c_void_p)
def eglQueryStreamMetadataNV(dpy,stream,name,n,offset,size,data):pass
@_f
@_p.types(_cs.EGLBoolean,_cs.EGLDisplay,_cs.EGLStreamKHR,_cs.EGLint,_cs.EGLint,_cs.EGLint,ctypes.c_void_p)
def eglSetStreamMetadataNV(dpy,stream,n,offset,size,data):pass



'''OpenGL extension KHR.texture_compression_astc_sliced_3d

This module customises the behaviour of the 
OpenGL.raw.GL.KHR.texture_compression_astc_sliced_3d to provide a more 
Python-friendly API

Overview (from the spec)
	
	Adaptive Scalable Texture Compression (ASTC) is a new texture
	compression technology that offers unprecendented flexibility, while
	producing better or comparable results than existing texture
	compressions at all bit rates. It includes support for 2D and
	slice-based 3D textures, with low and high dynamic range, at bitrates
	from below 1 bit/pixel up to 8 bits/pixel in fine steps.
	
	This extension extends the functionality of
	GL_KHR_texture_compression_astc_ldr to include slice-based 3D textures
	for textures using the LDR profile in the same way as the HDR profile
	allows slice-based 3D textures.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/texture_compression_astc_sliced_3d.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.KHR.texture_compression_astc_sliced_3d import *
from OpenGL.raw.GL.KHR.texture_compression_astc_sliced_3d import _EXTENSION_NAME

def glInitTextureCompressionAstcSliced3DKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.robustness_application_isolation

This module customises the behaviour of the 
OpenGL.raw.GLX.ARB.robustness_application_isolation to provide a more 
Python-friendly API

Overview (from the spec)
	
	GL_ARB_robustness and supporting window system extensions allow
	creating an OpenGL context supporting graphics reset notification
	behavior.  GL_ARB_robustness_isolation provides stronger
	guarantees about the possible side-effects of a graphics reset.
	
	It is expected that there may be a performance cost associated
	with isolating an application or share group from other contexts
	on the GPU. For this reason, GL_ARB_robustness_isolation is
	phrased as an opt-in mechanism, with a new context creation bit
	defined in the window system bindings. It is expected that
	implementations might only advertise the strings in this extension
	if both the implementation supports the desired isolation
	properties, and the context was created with the appropriate reset
	isolation bit.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/robustness_application_isolation.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.ARB.robustness_application_isolation import *
from OpenGL.raw.GLX.ARB.robustness_application_isolation import _EXTENSION_NAME

def glInitRobustnessApplicationIsolationARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_blend_func_extended'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_blend_func_extended',error_checker=_errors._error_checker)
GL_LOCATION_INDEX_EXT=_C('GL_LOCATION_INDEX_EXT',0x930F)
GL_MAX_DUAL_SOURCE_DRAW_BUFFERS_EXT=_C('GL_MAX_DUAL_SOURCE_DRAW_BUFFERS_EXT',0x88FC)
GL_ONE_MINUS_SRC1_ALPHA_EXT=_C('GL_ONE_MINUS_SRC1_ALPHA_EXT',0x88FB)
GL_ONE_MINUS_SRC1_COLOR_EXT=_C('GL_ONE_MINUS_SRC1_COLOR_EXT',0x88FA)
GL_SRC1_ALPHA_EXT=_C('GL_SRC1_ALPHA_EXT',0x8589)
GL_SRC1_COLOR_EXT=_C('GL_SRC1_COLOR_EXT',0x88F9)
GL_SRC_ALPHA_SATURATE_EXT=_C('GL_SRC_ALPHA_SATURATE_EXT',0x0308)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,arrays.GLcharArray)
def glBindFragDataLocationEXT(program,color,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,arrays.GLcharArray)
def glBindFragDataLocationIndexedEXT(program,colorNumber,index,name):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,arrays.GLcharArray)
def glGetFragDataIndexEXT(program,name):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetProgramResourceLocationIndexEXT(program,programInterface,name):pass



'''OpenGL extension NV.geometry_program4

This module customises the behaviour of the 
OpenGL.raw.GL.NV.geometry_program4 to provide a more 
Python-friendly API

Overview (from the spec)
	
	NV_geometry_program4 defines a new type of program available to be run on
	the GPU, called a geometry program.  Geometry programs are run on full
	primitives after vertices are transformed, but prior to flat shading and
	clipping.
	
	A geometry program begins with a single primitive - a point, line, or
	triangle.  Quads and polygons are allowed, but are decomposed into
	individual triangles prior to geometry program execution.  It can read the
	attributes of any of the vertex in the primitive and use them to generate
	new primitives.  A geometry program has a fixed output primitive type,
	either a point, a line strip, or a triangle strip.  It emits vertices
	(using the EMIT opcode) to define the output primitive.  The attributes of
	emitted vertices are specified by writing to the same set of result
	bindings (e.g., "result.position") provided for vertex programs.
	Additionally, a geometry program can emit multiple disconnected primitives
	by using the ENDPRIM opcode, which is roughly equivalent to calling End
	and then Begin again.  The primitives emitted by the geometry program are
	then clipped and then processed like an equivalent OpenGL primitive
	specified by the application.
	
	This extension provides four additional primitive types:  lines with
	adjacency, line strips with adjacency, separate triangles with adjacency,
	and triangle strips with adjacency.  Some of the vertices specified in
	these new primitive types are not part of the ordinary primitives.
	Instead, they represent neighboring vertices that are adjacent to the two
	line segment end points (lines/strips) or the three triangle edges
	(triangles/tstrips).  These "adjacency" vertices can be accessed by
	geometry programs and used to match up the outputs of the geometry program
	with those of neighboring primitives.
	
	Additionally, geometry programs allow for layered rendering, where entire
	three-dimensional, cube map, or array textures (EXT_texture_array) can be
	bound to the current framebuffer.  Geometry programs can use the
	"result.layer" binding to select a layer or cube map face to render to.
	Each primitive emitted by such a geometry program is rendered to the layer
	taken from its provoking vertex.
	
	Since geometry programs expect a specific input primitive type, an error
	will occur if the application presents primtives of a different type.  For
	example, if an enabled geometry program expects points, an error will
	occur at Begin() time, if a primitive mode of TRIANGLES is specified.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/geometry_program4.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.geometry_program4 import *
from OpenGL.raw.GL.NV.geometry_program4 import _EXTENSION_NAME

def glInitGeometryProgram4NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GLES3_3_1

This module customises the behaviour of the 
OpenGL.raw.GLES3.VERSION.GLES3_3_1 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GLES3_3_1.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES3 import _types, _glgets
from OpenGL.raw.GLES3.VERSION.GLES3_3_1 import *
from OpenGL.raw.GLES3.VERSION.GLES3_3_1 import _EXTENSION_NAME

def glInitGles331VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetFramebufferParameteriv=wrapper.wrapper(glGetFramebufferParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetProgramInterfaceiv=wrapper.wrapper(glGetProgramInterfaceiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glGetProgramResourceIndex.name size not checked against 'name'
glGetProgramResourceIndex=wrapper.wrapper(glGetProgramResourceIndex).setInputArraySize(
    'name', None
)
glGetProgramResourceName=wrapper.wrapper(glGetProgramResourceName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
# INPUT glGetProgramResourceiv.props size not checked against propCount
glGetProgramResourceiv=wrapper.wrapper(glGetProgramResourceiv).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setInputArraySize(
    'props', None
)
# INPUT glGetProgramResourceLocation.name size not checked against 'name'
glGetProgramResourceLocation=wrapper.wrapper(glGetProgramResourceLocation).setInputArraySize(
    'name', None
)
# INPUT glCreateShaderProgramv.strings size not checked against count
glCreateShaderProgramv=wrapper.wrapper(glCreateShaderProgramv).setInputArraySize(
    'strings', None
)
# INPUT glDeleteProgramPipelines.pipelines size not checked against n
glDeleteProgramPipelines=wrapper.wrapper(glDeleteProgramPipelines).setInputArraySize(
    'pipelines', None
)
glGenProgramPipelines=wrapper.wrapper(glGenProgramPipelines).setOutput(
    'pipelines',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGetProgramPipelineiv=wrapper.wrapper(glGetProgramPipelineiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glProgramUniform1iv.value size not checked against count
glProgramUniform1iv=wrapper.wrapper(glProgramUniform1iv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2iv.value size not checked against count*2
glProgramUniform2iv=wrapper.wrapper(glProgramUniform2iv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3iv.value size not checked against count*3
glProgramUniform3iv=wrapper.wrapper(glProgramUniform3iv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4iv.value size not checked against count*4
glProgramUniform4iv=wrapper.wrapper(glProgramUniform4iv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1uiv.value size not checked against count
glProgramUniform1uiv=wrapper.wrapper(glProgramUniform1uiv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2uiv.value size not checked against count*2
glProgramUniform2uiv=wrapper.wrapper(glProgramUniform2uiv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3uiv.value size not checked against count*3
glProgramUniform3uiv=wrapper.wrapper(glProgramUniform3uiv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4uiv.value size not checked against count*4
glProgramUniform4uiv=wrapper.wrapper(glProgramUniform4uiv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1fv.value size not checked against count
glProgramUniform1fv=wrapper.wrapper(glProgramUniform1fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2fv.value size not checked against count*2
glProgramUniform2fv=wrapper.wrapper(glProgramUniform2fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3fv.value size not checked against count*3
glProgramUniform3fv=wrapper.wrapper(glProgramUniform3fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4fv.value size not checked against count*4
glProgramUniform4fv=wrapper.wrapper(glProgramUniform4fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2fv.value size not checked against count*4
glProgramUniformMatrix2fv=wrapper.wrapper(glProgramUniformMatrix2fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3fv.value size not checked against count*9
glProgramUniformMatrix3fv=wrapper.wrapper(glProgramUniformMatrix3fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fv.value size not checked against count*16
glProgramUniformMatrix4fv=wrapper.wrapper(glProgramUniformMatrix4fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x3fv.value size not checked against count*6
glProgramUniformMatrix2x3fv=wrapper.wrapper(glProgramUniformMatrix2x3fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x2fv.value size not checked against count*6
glProgramUniformMatrix3x2fv=wrapper.wrapper(glProgramUniformMatrix3x2fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x4fv.value size not checked against count*8
glProgramUniformMatrix2x4fv=wrapper.wrapper(glProgramUniformMatrix2x4fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x2fv.value size not checked against count*8
glProgramUniformMatrix4x2fv=wrapper.wrapper(glProgramUniformMatrix4x2fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x4fv.value size not checked against count*12
glProgramUniformMatrix3x4fv=wrapper.wrapper(glProgramUniformMatrix3x4fv).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x3fv.value size not checked against count*12
glProgramUniformMatrix4x3fv=wrapper.wrapper(glProgramUniformMatrix4x3fv).setInputArraySize(
    'value', None
)
glGetProgramPipelineInfoLog=wrapper.wrapper(glGetProgramPipelineInfoLog).setOutput(
    'infoLog',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
glGetBooleani_v=wrapper.wrapper(glGetBooleani_v).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='target',orPassIn=True
)
glGetMultisamplefv=wrapper.wrapper(glGetMultisamplefv).setOutput(
    'val',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexLevelParameteriv=wrapper.wrapper(glGetTexLevelParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexLevelParameterfv=wrapper.wrapper(glGetTexLevelParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.transform_feedback2

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.transform_feedback2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	The EXT_transform_feedback extension allows applications to capture 
	primitives to one or more buffer objects when transformed by the GL.
	This extension provides a few additional capabilities to these extensions,
	making transform feedback mode more useful.
	
	First, it provides transform feedback objects which encapsulate transform
	feedback-related state, allowing applications to replace the entire
	transform feedback configuration in a single bind call.  Second, it
	provides the ability to pause and resume transform feedback operations.
	When transform feedback is paused, applications may render without
	transform feedback or may use transform feedback with different state and
	a different transform feedback object.  When transform feedback is
	resumed, additional primitives are captured and appended to previously
	captured primitives for the object.
	
	Additionally, this extension provides the ability to draw primitives
	captured in transform feedback mode without querying the captured
	primitive count.  The command DrawTransformFeedback() is equivalent to
	glDrawArrays(<mode>, 0, <count>), where <count> is the number of vertices
	captured to buffer objects during the last transform feedback capture
	operation on the transform feedback object used.  This draw operation only
	provides a vertex count -- it does not automatically set up vertex array
	state or vertex buffer object bindings, which must be done separately by
	the application.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/transform_feedback2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.transform_feedback2 import *
from OpenGL.raw.GL.ARB.transform_feedback2 import _EXTENSION_NAME

def glInitTransformFeedback2ARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDeleteTransformFeedbacks.ids size not checked against n
glDeleteTransformFeedbacks=wrapper.wrapper(glDeleteTransformFeedbacks).setInputArraySize(
    'ids', None
)
glGenTransformFeedbacks=wrapper.wrapper(glGenTransformFeedbacks).setOutput(
    'ids',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.multisampled_render_to_texture

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.multisampled_render_to_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces functionality to perform multisampled 
	rendering to a color renderable texture, without requiring an 
	explicit resolve of multisample data. 
	
	Some GPU architectures - such as tile-based renderers - are
	capable of performing multisampled rendering by storing
	multisample data in internal high-speed memory and downsampling the
	data when writing out to external memory after rendering has
	finished. Since per-sample data is never written out to external
	memory, this approach saves bandwidth and storage space. In this
	case multisample data gets discarded, however this is acceptable
	in most cases.
	
	The extension provides a new command, FramebufferTexture2DMultisampleEXT,
	which attaches a texture level to a framebuffer and enables
	multisampled rendering to that texture level.
	
	When the texture level is flushed or used as a source or destination
	for any operation other than drawing to it, an implicit resolve of
	multisampled color data may be performed. After such a resolve, the
	multisampled color data is discarded.
	
	In order to allow the use of multisampled depth and stencil buffers 
	when performing multisampled rendering to a texture, the extension 
	also adds the command RenderbufferStorageMultisampleEXT.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/multisampled_render_to_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.multisampled_render_to_texture import *
from OpenGL.raw.GLES2.EXT.multisampled_render_to_texture import _EXTENSION_NAME

def glInitMultisampledRenderToTextureEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.compressed_3DC_texture

This module customises the behaviour of the 
OpenGL.raw.GLES1.AMD.compressed_3DC_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	Two compression formats are introduced:
	
	- A compression format for two component textures.  When used to store 
	  normal vectors, the two components are commonly used with a fragment 
	  shader that derives the third component.
	
	- A compression format for single component textures.  The single component
	  may be used as a luminance or an alpha value.
	
	There are a large number of games that use luminance only and/or alpha only 
	textures.  For example, monochrome light maps used in a few popular games 
	are 8-bit luminance textures.  This extension describes a compression format
	that provides a 2:1 compression ratio for 8-bit single channel textures.
	
	Normal maps are special textures that are used to add detail to 3D surfaces.
	They are an extension of earlier "bump map" textures, which contained per-
	pixel height values and were used to create the appearance of bumpiness on 
	otherwise smooth surfaces.  Normal maps contain more detailed surface 
	information, allowing them to represent much more complex shapes.
	
	Normal mapping is one of the key features that makes the current generation
	of games look so much better than earlier titles.  A limitation to the 
	effectiveness of this technique is the size of the textures required.  In an 
	ideal case where every surface has both a color texture map and a normal 
	texture map, the texture memory and bandwidth requirements would double 
	compared to using color maps alone.
	
	In fact, the problem is much worse because existing block based compression
	methods such as DXTc, ETC, and S3TC are ineffective at compressing normal 
	maps.  They tend to have trouble capturing the small edges and subtle 
	curvature that normal maps are designed to capture, and they also introduce 
	unsightly block artifacts.
	
	Because normal maps are used to capture light reflections and realistic 
	surface highlights, these problems are amplified relative to their impact on
	color textures.  The results are sufficiently poor that game artists and 
	developers would rather not use normal map compression at all on most 
	surfaces, and instead limit themselves to lower resolution maps on selected 
	parts of the rendered scene.
	
	3DC provides an ideal solution to the normal map compression problem.  It 
	provides up to 4:1 compression of normal maps, with image quality that is 
	virtually indistinguishable from the uncompressed version.  The technique is
	hardware accelerated, so the performance impact is minimal.  Thus, 
	developers are freed to use higher resolution, more detailed normal maps, 
	and/or use them on all of the objects in a scene rather than just a select 
	few.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/compressed_3DC_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.AMD.compressed_3DC_texture import *
from OpenGL.raw.GLES1.AMD.compressed_3DC_texture import _EXTENSION_NAME

def glInitCompressed3DcTextureAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_clear_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_clear_texture',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearTexImageEXT(texture,level,format,type,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearTexSubImageEXT(texture,level,xoffset,yoffset,zoffset,width,height,depth,format,type,data):pass



'''OpenGL extension AMD.pinned_memory

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.pinned_memory to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines an interface that allows improved control
	of the physical memory used by the graphics device.
	
	It allows an existing page of system memory allocated by the application
	to be used as memory directly accessible to the graphics processor. One
	example application of this functionality would be to be able to avoid an
	explicit synchronous copy with sub-system of the application; for instance
	it is possible to directly draw from a system memory copy of a video
	image.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/pinned_memory.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.pinned_memory import *
from OpenGL.raw.GL.AMD.pinned_memory import _EXTENSION_NAME

def glInitPinnedMemoryAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_format_BGRA8888

This module customises the behaviour of the 
OpenGL.raw.GLES1.EXT.texture_format_BGRA8888 to provide a more 
Python-friendly API

Overview (from the spec)
	
	 This extension provides an additional format and type combination
	 for use when specifying texture data.  The current allowed combinations
	 are:
	
	 Internal Format     External Format Type                    Bytes per Pixel
	 ---------------     --------------- ----                    ---------------
	 RGBA                RGBA             UNSIGNED_BYTE          4
	 RGB                 RGB              UNSIGNED_BYTE          3
	 RGBA                RGBA             UNSIGNED_SHORT_4_4_4_4 2
	 RGBA                RGBA             UNSIGNED_SHORT_5_5_5_1 2
	 RGB                 RGB              UNSIGNED_SHORT_5_6_5   2
	 LUMINANCE_ALPHA     LUMINANCE_ALPHA  UNSIGNED_BYTE          2
	 LUMINANCE           LUMINANCE        UNSIGNED_BYTE          1
	 ALPHA               ALPHA            UNSIGNED_BYTE          1
	
	
	This table is extended to include format BGRA_EXT and type UNSIGNED_BYTE:
	
	 Internal Format     External Format Type                    Bytes per Pixel
	 ---------------     --------------- ----                    ---------------
	 BGRA_EXT            BGRA_EXT        UNSIGNED_BYTE           4
	 RGBA                RGBA            UNSIGNED_BYTE           4
	 RGB                 RGB             UNSIGNED_BYTE           3
	 RGBA                RGBA            UNSIGNED_SHORT_4_4_4_4  2
	 RGBA                RGBA            UNSIGNED_SHORT_5_5_5_1  2
	 RGB                 RGB             UNSIGNED_SHORT_5_6_5    2
	 LUMINANCE_ALPHA     LUMINANCE_ALPHA UNSIGNED_BYTE           2
	 LUMINANCE           LUMINANCE       UNSIGNED_BYTE           1
	 ALPHA               ALPHA           UNSIGNED_BYTE           1
	
	 This format is renderable in versions of OpenGL ES from 2.0 onwards.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_format_BGRA8888.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.EXT.texture_format_BGRA8888 import *
from OpenGL.raw.GLES1.EXT.texture_format_BGRA8888 import _EXTENSION_NAME

def glInitTextureFormatBgra8888EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.EGL_image_array

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.EGL_image_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds functionality to that provided by OES_EGL_image in
	order to support EGLImage 2D arrays. It extends the existing
	EGLImageTargetTexture2DOES entry point from OES_EGL_image. Render buffers
	are not extended to include array support.
	
	EGLImage 2D arrays can be created using extended versions of eglCreateImageKHR.
	For example, EGL_ANDROID_image_native_buffer can import image array native buffers
	on devices where such native buffers can be created.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/EGL_image_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.EGL_image_array import *
from OpenGL.raw.GLES2.EXT.EGL_image_array import _EXTENSION_NAME

def glInitEglImageArrayEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.texture_filter_anisotropic

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_filter_anisotropic to provide a more 
Python-friendly API

Overview (from the spec)
	
	Texture mapping using OpenGL's existing mipmap texture filtering
	modes assumes that the projection of the pixel filter footprint into
	texture space is a square (ie, isotropic).  In practice however, the
	footprint may be long and narrow (ie, anisotropic).  Consequently,
	mipmap filtering severely blurs images on surfaces angled obliquely
	away from the viewer.
	
	Several approaches exist for improving texture sampling by accounting
	for the anisotropic nature of the pixel filter footprint into texture
	space.  This extension provides a general mechanism for supporting
	anisotropic texturing filtering schemes without specifying a
	particular formulation of anisotropic filtering.
	
	The extension permits the OpenGL application to specify on a per-texture
	or -sampler object basis the maximum degree of anisotropy to account for
	in texture filtering.
	
	Increasing the maximum degree of anisotropy may
	improve texture filtering, but may also significantly reduce the
	implementation's texture filtering rate.  Implementations are free
	to clamp the specified degree of anisotropy to the implementation's
	maximum supported degree of anisotropy.
	
	A sampler or texture's maximum degree of anisotropy is specified
	independently from its minification and magnification filter (as
	opposed to being supported as an entirely new filtering mode).
	Implementations are free to use the specified minification and
	magnification filter to select a particular anisotropic texture
	filtering scheme.  For example, a NEAREST filter with a maximum
	degree of anisotropy of two could be treated as a 2-tap filter that
	accounts for the direction of anisotropy.  Implementations are also
	permitted to ignore the minification or magnification filter and
	implement the highest quality of anisotropic filtering possible.
	
	Applications seeking the highest quality anisotropic filtering
	available are advised to request a LINEAR_MIPMAP_LINEAR minification
	filter, a LINEAR magnification filter, and a large maximum degree
	of anisotropy.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_filter_anisotropic.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_filter_anisotropic import *
from OpenGL.raw.GL.ARB.texture_filter_anisotropic import _EXTENSION_NAME

def glInitTextureFilterAnisotropicARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.YUV_target

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.YUV_target to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for three new YUV related items: first
	rendering to YUV images, second sampling from YUV images while keeping the
	data in YUV space, third it defines a new built in function that does
	conversion from RGB to YUV with controls to choose ITU-R BT.601-7,
	ITU-R BT.601-7 Full range (JFIF images), or ITU-R BT.709-5 standard.
	
	This new functionality is layered on top of the OES_EGL_image_external
	extension.
	
	To perform the YUV rendering capability in this extension an application
	will attach a texture to the framebuffer object as the color attachment.
	If the texture has a target type of TEXTURE_EXTERNAL_OES with YUV color
	format then the GL driver can use this framebuffer object as the render
	target, TEXTURE_EXTERNAL_OES target with RGB color format are not allowed
	with this extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/YUV_target.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.YUV_target import *
from OpenGL.raw.GLES2.EXT.YUV_target import _EXTENSION_NAME

def glInitYuvTargetEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.query_resource

This module customises the behaviour of the 
OpenGL.raw.GL.NV.query_resource to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL implementations manage the residence of textures, shaders, and
	other graphical objects in GPU accessible memory (whether in on-board
	video memory or addressable system memory is implementation dependent).
	With more insight into OpenGL's memory usage 1) applications could make
	educated decisions on better utilizing the limited GPU resources,
	2) users could better optimize their workflow when working with multiple
	tools, and 3) administrators can make better decisions regarding resource
	allocation and system configurations.
	
	The purpose of this extension is to return a more detailed breakdown
	of memory usage in terms of the OpenGL objects residing in memory
	(textures, render buffers, buffer objects, system reserved objects, ...).
	This extension differs from GL_NVX_gpu_memory_info in that this extension
	returns detailed memory usage at the object level for video memory while
	the other extension only reports total vidmem usage.
	
	For the purposes of this specification the term vidmem refers to video
	memory resident on the graphics card that is directly accessible to the
	GPU at the highest performance level.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/query_resource.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.query_resource import *
from OpenGL.raw.GL.NV.query_resource import _EXTENSION_NAME

def glInitQueryResourceNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.blend_minmax_factor

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.blend_minmax_factor to provide a more 
Python-friendly API

Overview (from the spec)
	
	The EXT_blend_minmax extension extended the GL's blending
	functionality to allow the blending equation to be specified by the
	application. That extension introduced the MIN_EXT and MAX_EXT blend
	equations, which caused the result of the blend equation to become
	the minimum or maximum of the source color and destination color,
	respectively.
	
	The MIN_EXT and MAX_EXT blend equations, however, do not include the
	source or destination blend factors in the arguments to the min and
	max functions.  This extension provides two new blend equations that
	produce the minimum or maximum of the products of the source color
	and source factor, and the destination color and destination factor.
	
	This NVIDIA extension has some limitations relative to the
	AMD_blend_minmax_factor extension.  See issues #1, #2, and #3.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/blend_minmax_factor.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.blend_minmax_factor import *
from OpenGL.raw.GLES2.NV.blend_minmax_factor import _EXTENSION_NAME

def glInitBlendMinmaxFactorNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ES.VERSION_3_2

This module customises the behaviour of the 
OpenGL.raw.GLES2.ES.VERSION_3_2 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ES/VERSION_3_2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ES.VERSION_3_2 import *
from OpenGL.raw.GLES2.ES.VERSION_3_2 import _EXTENSION_NAME

def glInitVersion32ES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDebugMessageControl.ids size not checked against count
glDebugMessageControl=wrapper.wrapper(glDebugMessageControl).setInputArraySize(
    'ids', None
)
# INPUT glDebugMessageInsert.buf size not checked against 'buf,length'
glDebugMessageInsert=wrapper.wrapper(glDebugMessageInsert).setInputArraySize(
    'buf', None
)
glGetDebugMessageLog=wrapper.wrapper(glGetDebugMessageLog).setOutput(
    'ids',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'lengths',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'messageLog',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'severities',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'sources',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'types',size=lambda x:(x,),pnameArg='count',orPassIn=True
)
# INPUT glPushDebugGroup.message size not checked against 'message,length'
glPushDebugGroup=wrapper.wrapper(glPushDebugGroup).setInputArraySize(
    'message', None
)
# INPUT glObjectLabel.label size not checked against 'label,length'
glObjectLabel=wrapper.wrapper(glObjectLabel).setInputArraySize(
    'label', None
)
glGetObjectLabel=wrapper.wrapper(glGetObjectLabel).setOutput(
    'label',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
# INPUT glObjectPtrLabel.label size not checked against 'label,length'
glObjectPtrLabel=wrapper.wrapper(glObjectPtrLabel).setInputArraySize(
    'label', None
)
glGetObjectPtrLabel=wrapper.wrapper(glGetObjectPtrLabel).setOutput(
    'label',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'length',size=(1,),orPassIn=True
)
glGetPointerv=wrapper.wrapper(glGetPointerv).setOutput(
    'params',size=(1,),orPassIn=True
)
# INPUT glDrawElementsBaseVertex.indices size not checked against 'count,type'
glDrawElementsBaseVertex=wrapper.wrapper(glDrawElementsBaseVertex).setInputArraySize(
    'indices', None
)
# INPUT glDrawRangeElementsBaseVertex.indices size not checked against 'count,type'
glDrawRangeElementsBaseVertex=wrapper.wrapper(glDrawRangeElementsBaseVertex).setInputArraySize(
    'indices', None
)
# INPUT glDrawElementsInstancedBaseVertex.indices size not checked against 'count,type'
glDrawElementsInstancedBaseVertex=wrapper.wrapper(glDrawElementsInstancedBaseVertex).setInputArraySize(
    'indices', None
)
# INPUT glReadnPixels.data size not checked against bufSize
glReadnPixels=wrapper.wrapper(glReadnPixels).setInputArraySize(
    'data', None
)
# INPUT glGetnUniformfv.params size not checked against bufSize
glGetnUniformfv=wrapper.wrapper(glGetnUniformfv).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformiv.params size not checked against bufSize
glGetnUniformiv=wrapper.wrapper(glGetnUniformiv).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformuiv.params size not checked against bufSize
glGetnUniformuiv=wrapper.wrapper(glGetnUniformuiv).setInputArraySize(
    'params', None
)
# INPUT glTexParameterIiv.params size not checked against 'pname'
glTexParameterIiv=wrapper.wrapper(glTexParameterIiv).setInputArraySize(
    'params', None
)
# INPUT glTexParameterIuiv.params size not checked against 'pname'
glTexParameterIuiv=wrapper.wrapper(glTexParameterIuiv).setInputArraySize(
    'params', None
)
glGetTexParameterIiv=wrapper.wrapper(glGetTexParameterIiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexParameterIuiv=wrapper.wrapper(glGetTexParameterIuiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glSamplerParameterIiv.param size not checked against 'pname'
glSamplerParameterIiv=wrapper.wrapper(glSamplerParameterIiv).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterIuiv.param size not checked against 'pname'
glSamplerParameterIuiv=wrapper.wrapper(glSamplerParameterIuiv).setInputArraySize(
    'param', None
)
glGetSamplerParameterIiv=wrapper.wrapper(glGetSamplerParameterIiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetSamplerParameterIuiv=wrapper.wrapper(glGetSamplerParameterIuiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.draw_transform_feedback

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.draw_transform_feedback to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the DrawTransformFeedback commands. These
	were omitted from OpenGL ES 3.0 because the number of vertices
	captured by transform feedback could never be different than
	the number drawn during capture. The addition of geometry shaders
	in OpenGL ES 3.2 broke that assumption but, due to an oversight,
	DrawTransformFeedback et al were not reinstated. The
	DrawTransformFeedback commands unlock the full potential of
	geometry shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/draw_transform_feedback.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.draw_transform_feedback import *
from OpenGL.raw.GLES2.EXT.draw_transform_feedback import _EXTENSION_NAME

def glInitDrawTransformFeedbackEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_gpu_shader_int64'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_gpu_shader_int64',error_checker=_errors._error_checker)
GL_INT64_ARB=_C('GL_INT64_ARB',0x140E)
GL_INT64_VEC2_ARB=_C('GL_INT64_VEC2_ARB',0x8FE9)
GL_INT64_VEC3_ARB=_C('GL_INT64_VEC3_ARB',0x8FEA)
GL_INT64_VEC4_ARB=_C('GL_INT64_VEC4_ARB',0x8FEB)
GL_UNSIGNED_INT64_ARB=_C('GL_UNSIGNED_INT64_ARB',0x140F)
GL_UNSIGNED_INT64_VEC2_ARB=_C('GL_UNSIGNED_INT64_VEC2_ARB',0x8FF5)
GL_UNSIGNED_INT64_VEC3_ARB=_C('GL_UNSIGNED_INT64_VEC3_ARB',0x8FF6)
GL_UNSIGNED_INT64_VEC4_ARB=_C('GL_UNSIGNED_INT64_VEC4_ARB',0x8FF7)
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLint64Array)
def glGetUniformi64vARB(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLuint64Array)
def glGetUniformui64vARB(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glGetnUniformi64vARB(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glGetnUniformui64vARB(program,location,bufSize,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64)
def glProgramUniform1i64ARB(program,location,x):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform1i64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64)
def glProgramUniform1ui64ARB(program,location,x):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform1ui64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64,_cs.GLint64)
def glProgramUniform2i64ARB(program,location,x,y):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform2i64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64,_cs.GLuint64)
def glProgramUniform2ui64ARB(program,location,x,y):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform2ui64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64,_cs.GLint64,_cs.GLint64)
def glProgramUniform3i64ARB(program,location,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform3i64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64)
def glProgramUniform3ui64ARB(program,location,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform3ui64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64,_cs.GLint64,_cs.GLint64,_cs.GLint64)
def glProgramUniform4i64ARB(program,location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform4i64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64)
def glProgramUniform4ui64ARB(program,location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform4ui64vARB(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64)
def glUniform1i64ARB(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform1i64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64)
def glUniform1ui64ARB(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform1ui64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64,_cs.GLint64)
def glUniform2i64ARB(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform2i64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64,_cs.GLuint64)
def glUniform2ui64ARB(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform2ui64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64,_cs.GLint64,_cs.GLint64)
def glUniform3i64ARB(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform3i64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64)
def glUniform3ui64ARB(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform3ui64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64,_cs.GLint64,_cs.GLint64,_cs.GLint64)
def glUniform4i64ARB(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform4i64vARB(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64,_cs.GLuint64)
def glUniform4ui64ARB(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform4ui64vARB(location,count,value):pass



'''OpenGL extension KHR.texture_compression_astc_hdr

This module customises the behaviour of the 
OpenGL.raw.GL.KHR.texture_compression_astc_hdr to provide a more 
Python-friendly API

Overview (from the spec)
	
	Adaptive Scalable Texture Compression (ASTC) is a new texture
	compression technology that offers unprecendented flexibility, while
	producing better or comparable results than existing texture
	compressions at all bit rates. It includes support for 2D and
	slice-based 3D textures, with low and high dynamic range, at bitrates
	from below 1 bit/pixel up to 8 bits/pixel in fine steps.
	
	The goal of these extensions is to support the full 2D profile of the
	ASTC texture compression specification, and allow construction of 3D
	textures from multiple compressed 2D slices.
	
	ASTC-compressed textures are handled in OpenGL ES and OpenGL by adding
	new supported formats to the existing commands for defining and updating
	compressed textures, and defining the interaction of the ASTC formats
	with each texture target.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/texture_compression_astc_hdr.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.KHR.texture_compression_astc_hdr import *
from OpenGL.raw.GL.KHR.texture_compression_astc_hdr import _EXTENSION_NAME

def glInitTextureCompressionAstcHdrKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_format_sRGB_override

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_format_sRGB_override to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new texture parameter to override the internal
	format of a texture object; allowing a non-sRGB format to be overridden to
	a corresponding sRGB format.  For example, an RGB8 texture can be overridden
	to SRGB8.  Such an override will cause the RGB components to be "decoded" from
	sRGB color space to linear as part of texture filtering.  This can be useful for
	applications where a texture was written with sRGB data using EXT_sRGB_write_control
	or when sampling from an EGLImage that is known to contain sRGB color values.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_format_sRGB_override.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_format_sRGB_override import *
from OpenGL.raw.GLES2.EXT.texture_format_sRGB_override import _EXTENSION_NAME

def glInitTextureFormatSrgbOverrideEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.fragment_precision_high

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.fragment_precision_high to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension has been withdrawn. See the specification of
	GetShaderPrecisionFormat in section 6.1.8 of the OpenGL ES 2.0
	Specification to determine within the API if high-precision fragment
	shader varyings are supported by the implementation.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/fragment_precision_high.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.fragment_precision_high import *
from OpenGL.raw.GLES2.OES.fragment_precision_high import _EXTENSION_NAME

def glInitFragmentPrecisionHighOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.stencil_operation_extended

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.stencil_operation_extended to provide a more 
Python-friendly API

Overview (from the spec)
	
	Stencil buffers are special buffers that allow tests to be made against an
	incoming value and action taken based on that value. The stencil buffer is
	updated during rasterization, and the operation used to update the stencil
	buffer is chosen based on whether the fragment passes the stencil test,
	and if it does, whether it passes the depth test. Traditional OpenGL
	includes support for several primitive operations, such as incrementing,
	or clearing the content of the stencil buffer, or replacing it with a
	specified reference value.
	
	This extension adds support for an additional set of operations that may
	be performed on the stencil buffer under each circumstance. Additionally,
	this extension separates the value used as the source for stencil
	operations from the reference value, allowing different values to be used
	in the stencil test, and in the update of the stencil buffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/stencil_operation_extended.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.stencil_operation_extended import *
from OpenGL.raw.GL.AMD.stencil_operation_extended import _EXTENSION_NAME

def glInitStencilOperationExtendedAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NVX.blend_equation_advanced_multi_draw_buffers

This module customises the behaviour of the 
OpenGL.raw.GL.NVX.blend_equation_advanced_multi_draw_buffers to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for using advanced blend equations
	introduced with NV_blend_equation_advanced (and standardized
	by KHR_blend_equation_advanced) in conjunction with multiple
	draw buffers.  The NV_blend_equation_advanced extension supports
	advanced blending equations only when rending to a single color
	buffer using fragment color zero and throws and INVALID_OPERATION
	error when multiple draw buffers are used. This extension removes
	this restriction.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NVX/blend_equation_advanced_multi_draw_buffers.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NVX.blend_equation_advanced_multi_draw_buffers import *
from OpenGL.raw.GL.NVX.blend_equation_advanced_multi_draw_buffers import _EXTENSION_NAME

def glInitBlendEquationAdvancedMultiDrawBuffersNVX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.viewport_swizzle

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.viewport_swizzle to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new per-viewport swizzle that can modify the
	position of primitives sent to each viewport.  New viewport swizzle state
	is added for each viewport, and a new position vector is computed for each
	vertex by selecting from and optionally negating any of the four
	components of the original position vector.
	
	This new viewport swizzle is useful for a number of algorithms, including
	single-pass cubemap rendering (broadcasting a primitive to multiple faces
	and reorienting the vertex position for each face) and voxel
	rasterization.  The per-viewport component remapping and negation provided
	by the swizzle allows application code to re-orient three-dimensional
	geometry with a view along any of the X, Y, or Z axes.  If a perspective
	projection and depth buffering is required, 1/W buffering should be used,
	as described in the single-pass cubemap rendering example in the "Issues"
	section below.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/viewport_swizzle.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.viewport_swizzle import *
from OpenGL.raw.GLES2.NV.viewport_swizzle import _EXTENSION_NAME

def glInitViewportSwizzleNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_sparse_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_sparse_texture',error_checker=_errors._error_checker)
GL_MAX_SPARSE_3D_TEXTURE_SIZE_EXT=_C('GL_MAX_SPARSE_3D_TEXTURE_SIZE_EXT',0x9199)
GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS_EXT=_C('GL_MAX_SPARSE_ARRAY_TEXTURE_LAYERS_EXT',0x919A)
GL_MAX_SPARSE_TEXTURE_SIZE_EXT=_C('GL_MAX_SPARSE_TEXTURE_SIZE_EXT',0x9198)
GL_NUM_SPARSE_LEVELS_EXT=_C('GL_NUM_SPARSE_LEVELS_EXT',0x91AA)
GL_NUM_VIRTUAL_PAGE_SIZES_EXT=_C('GL_NUM_VIRTUAL_PAGE_SIZES_EXT',0x91A8)
GL_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS_EXT=_C('GL_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS_EXT',0x91A9)
GL_TEXTURE_2D=_C('GL_TEXTURE_2D',0x0DE1)
GL_TEXTURE_2D_ARRAY=_C('GL_TEXTURE_2D_ARRAY',0x8C1A)
GL_TEXTURE_3D=_C('GL_TEXTURE_3D',0x806F)
GL_TEXTURE_CUBE_MAP=_C('GL_TEXTURE_CUBE_MAP',0x8513)
GL_TEXTURE_CUBE_MAP_ARRAY_OES=_C('GL_TEXTURE_CUBE_MAP_ARRAY_OES',0x9009)
GL_TEXTURE_SPARSE_EXT=_C('GL_TEXTURE_SPARSE_EXT',0x91A6)
GL_VIRTUAL_PAGE_SIZE_INDEX_EXT=_C('GL_VIRTUAL_PAGE_SIZE_INDEX_EXT',0x91A7)
GL_VIRTUAL_PAGE_SIZE_X_EXT=_C('GL_VIRTUAL_PAGE_SIZE_X_EXT',0x9195)
GL_VIRTUAL_PAGE_SIZE_Y_EXT=_C('GL_VIRTUAL_PAGE_SIZE_Y_EXT',0x9196)
GL_VIRTUAL_PAGE_SIZE_Z_EXT=_C('GL_VIRTUAL_PAGE_SIZE_Z_EXT',0x9197)
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexPageCommitmentEXT(target,level,xoffset,yoffset,zoffset,width,height,depth,commit):pass



'''OpenGL extension EXT.instanced_arrays

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.instanced_arrays to provide a more 
Python-friendly API

Overview (from the spec)
	
	A common use case in GL for some applications is to be able to
	draw the same object, or groups of similar objects that share
	vertex data, primitive count and type, multiple times.  This 
	extension provides a means of accelerating such use cases while 
	reducing the number of API calls, and keeping the amount of 
	duplicate data to a minimum.
	
	This extension introduces an array "divisor" for generic
	vertex array attributes, which when non-zero specifies that the
	attribute is "instanced."  An instanced attribute does not
	advance per-vertex as usual, but rather after every <divisor>
	conceptual draw calls.
	
	(Attributes which aren't instanced are repeated in their entirety
	for every conceptual draw call.)
	
	By specifying transform data in an instanced attribute or series
	of instanced attributes, vertex shaders can, in concert with the 
	instancing draw calls, draw multiple instances of an object with 
	one draw call.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/instanced_arrays.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.instanced_arrays import *
from OpenGL.raw.GLES2.EXT.instanced_arrays import _EXTENSION_NAME

def glInitInstancedArraysEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawElementsInstancedEXT.indices size not checked against 'count,type'
glDrawElementsInstancedEXT=wrapper.wrapper(glDrawElementsInstancedEXT).setInputArraySize(
    'indices', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.device_base

This module customises the behaviour of the 
OpenGL.raw.EGL.EXT.device_base to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/device_base.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.EGL import _types, _glgets
from OpenGL.raw.EGL.EXT.device_base import *
from OpenGL.raw.EGL.EXT.device_base import _EXTENSION_NAME

def glInitDeviceBaseEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION
eglQueryDevicesEXT.extension = None 
eglQueryDevicesEXT.force_extension = True 
eglQueryDeviceStringEXT.extension = None
eglQueryDeviceStringEXT.force_extension = True 


def egl_get_devices(max_count=10):
    """Utility function that retrieves platform devices"""
    devices = (_types.EGLDeviceEXT*max_count)()
    count = _types.EGLint()
    if eglQueryDevicesEXT(max_count,devices,count):
        return devices[:count.value]
    return []



'''OpenGL extension EXT.clip_control

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.clip_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides additional clip control modes to configure how
	clip space is mapped to window space.  This extension's goal is to 1)
	allow OpenGL to effectively match Direct3D's coordinate system
	conventions, and 2) potentially improve the numerical precision of the Z
	coordinate mapping.
	
	This extension is a port of GL_ARB_clip_control to OpenGL ES. For the
	complete overview of this extension, refer to the "Overview" section of
	GL_ARB_clip_control.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/clip_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.clip_control import *
from OpenGL.raw.GLES2.EXT.clip_control import _EXTENSION_NAME

def glInitClipControlEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_compression_astc_decode_mode

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_compression_astc_decode_mode to provide a more 
Python-friendly API

Overview (from the spec)
	
	Adaptive Scalable Texture Compression (ASTC) is a texture compression
	technology that is exposed by existing extensions and specifications.
	
	The existing specifications require that low dynamic range (LDR)
	textures are decompressed to FP16 values per component. In many cases,
	decompressing LDR textures to a lower precision intermediate result gives
	acceptable image quality. Source material for LDR textures is typically
	authored as 8-bit UNORM values, so decoding to FP16 values adds little
	value. On the other hand, reducing precision of the decoded result
	reduces the size of the decompressed data, potentially improving texture
	cache performance and saving power.
	
	The goal of this extension is to enable this efficiency gain on existing
	ASTC texture data. This is achieved by giving the application the ability
	to select the decoding precision.
	
	Two decoding options are provided by
	GL_EXT_texture_compression_astc_decode_mode
	 - Decode to FP16: This is the default, and matches the required behavior
	   in existing APIs.
	 - Decode to UNORM8: This is provided as an option in LDR mode.
	
	If GL_EXT_texture_compression_astc_decode_mode_rgb9e5 is supported, then
	a third decoding option is provided:
	 - Decode to RGB9_E5: This is provided as an option in both LDR and HDR
	   mode. In this mode, negative values cannot be represented and are
	   clamped to zero. The alpha component is ignored, and the results
	   are as if alpha was 1.0.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_compression_astc_decode_mode.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_compression_astc_decode_mode import *
from OpenGL.raw.GLES2.EXT.texture_compression_astc_decode_mode import _EXTENSION_NAME

def glInitTextureCompressionAstcDecodeModeEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension APPLE.sync

This module customises the behaviour of the 
OpenGL.raw.GLES1.APPLE.sync to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces the concept of "sync objects". Sync
	objects are a synchronization primitive - a representation of events
	whose completion status can be tested or waited upon. One specific
	type of sync object, the "fence sync object", is supported in this
	extension, and additional types can easily be added in the future.
	
	Fence sync objects have corresponding fences, which are inserted
	into the OpenGL command stream at the time the sync object is
	created. A sync object can be queried for a given condition. The
	only condition supported for fence sync objects is completion of the
	corresponding fence command. Fence completion allows applications to
	request a partial Finish, wherein all commands prior to the fence
	will be forced to complete before control is returned to the calling
	process.
	
	These new mechanisms allow for synchronization between the host CPU
	and the GPU, which may be accessing the same resources (typically
	memory), as well as between multiple GL contexts bound to multiple
	threads in the host CPU.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/sync.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.APPLE.sync import *
from OpenGL.raw.GLES1.APPLE.sync import _EXTENSION_NAME

def glInitSyncAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetSyncivAPPLE.values size not checked against bufSize
glGetSyncivAPPLE=wrapper.wrapper(glGetSyncivAPPLE).setInputArraySize(
    'values', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.ycrcb_subsample

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.ycrcb_subsample to provide a more 
Python-friendly API

Overview (from the spec)
	
	(Need to construct a real extension spec based on this)

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/ycrcb_subsample.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.ycrcb_subsample import *
from OpenGL.raw.GL.SGIX.ycrcb_subsample import _EXTENSION_NAME

def glInitYcrcbSubsampleSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.robustness_video_memory_purge

This module customises the behaviour of the 
OpenGL.raw.GL.NV.robustness_video_memory_purge to provide a more 
Python-friendly API

Overview (from the spec)
	
	Allow applications to be notified when video memory has been purged.
	
	The NVIDIA OpenGL driver architecture on Linux has a limitation:
	resources located in video memory are not persistent across certain
	events. VT switches, suspend/resume events, and mode switching
	events may erase the contents of video memory. Any resource that
	is located exclusively in video memory, such as framebuffer objects
	(FBOs), will be lost. As the OpenGL specification makes no mention
	of events where the video memory is allowed to be cleared, the
	driver attempts to hide this fact from the application, but cannot
	do it for all resources.
	
	This extension provides a way for applications to discover when video
	memory content has been lost, so that the application can re-populate
	the video memory content as necessary.
	
	This extension will have a limited lifespan, as planned architectural
	evolutions in the NVIDIA Linux driver stack will allow
	video memory to be persistent. Any driver that exposes this
	extension is a driver that considers video memory to be
	volatile. Once the driver stack has been improved, the extension
	will no longer be exposed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/robustness_video_memory_purge.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.robustness_video_memory_purge import *
from OpenGL.raw.GL.NV.robustness_video_memory_purge import _EXTENSION_NAME

def glInitRobustnessVideoMemoryPurgeNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_vertex_buffer_unified_memory'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_vertex_buffer_unified_memory',error_checker=_errors._error_checker)
GL_COLOR_ARRAY_ADDRESS_NV=_C('GL_COLOR_ARRAY_ADDRESS_NV',0x8F23)
GL_COLOR_ARRAY_LENGTH_NV=_C('GL_COLOR_ARRAY_LENGTH_NV',0x8F2D)
GL_DRAW_INDIRECT_ADDRESS_NV=_C('GL_DRAW_INDIRECT_ADDRESS_NV',0x8F41)
GL_DRAW_INDIRECT_LENGTH_NV=_C('GL_DRAW_INDIRECT_LENGTH_NV',0x8F42)
GL_DRAW_INDIRECT_UNIFIED_NV=_C('GL_DRAW_INDIRECT_UNIFIED_NV',0x8F40)
GL_EDGE_FLAG_ARRAY_ADDRESS_NV=_C('GL_EDGE_FLAG_ARRAY_ADDRESS_NV',0x8F26)
GL_EDGE_FLAG_ARRAY_LENGTH_NV=_C('GL_EDGE_FLAG_ARRAY_LENGTH_NV',0x8F30)
GL_ELEMENT_ARRAY_ADDRESS_NV=_C('GL_ELEMENT_ARRAY_ADDRESS_NV',0x8F29)
GL_ELEMENT_ARRAY_LENGTH_NV=_C('GL_ELEMENT_ARRAY_LENGTH_NV',0x8F33)
GL_ELEMENT_ARRAY_UNIFIED_NV=_C('GL_ELEMENT_ARRAY_UNIFIED_NV',0x8F1F)
GL_FOG_COORD_ARRAY_ADDRESS_NV=_C('GL_FOG_COORD_ARRAY_ADDRESS_NV',0x8F28)
GL_FOG_COORD_ARRAY_LENGTH_NV=_C('GL_FOG_COORD_ARRAY_LENGTH_NV',0x8F32)
GL_INDEX_ARRAY_ADDRESS_NV=_C('GL_INDEX_ARRAY_ADDRESS_NV',0x8F24)
GL_INDEX_ARRAY_LENGTH_NV=_C('GL_INDEX_ARRAY_LENGTH_NV',0x8F2E)
GL_NORMAL_ARRAY_ADDRESS_NV=_C('GL_NORMAL_ARRAY_ADDRESS_NV',0x8F22)
GL_NORMAL_ARRAY_LENGTH_NV=_C('GL_NORMAL_ARRAY_LENGTH_NV',0x8F2C)
GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV=_C('GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV',0x8F27)
GL_SECONDARY_COLOR_ARRAY_LENGTH_NV=_C('GL_SECONDARY_COLOR_ARRAY_LENGTH_NV',0x8F31)
GL_TEXTURE_COORD_ARRAY_ADDRESS_NV=_C('GL_TEXTURE_COORD_ARRAY_ADDRESS_NV',0x8F25)
GL_TEXTURE_COORD_ARRAY_LENGTH_NV=_C('GL_TEXTURE_COORD_ARRAY_LENGTH_NV',0x8F2F)
GL_VERTEX_ARRAY_ADDRESS_NV=_C('GL_VERTEX_ARRAY_ADDRESS_NV',0x8F21)
GL_VERTEX_ARRAY_LENGTH_NV=_C('GL_VERTEX_ARRAY_LENGTH_NV',0x8F2B)
GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV=_C('GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV',0x8F20)
GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV=_C('GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV',0x8F2A)
GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV=_C('GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV',0x8F1E)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint64EXT,_cs.GLsizeiptr)
def glBufferAddressRangeNV(pname,index,address,length):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei)
def glColorFormatNV(size,type,stride):pass
@_f
@_p.types(None,_cs.GLsizei)
def glEdgeFlagFormatNV(stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei)
def glFogCoordFormatNV(type,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLuint64Array)
def glGetIntegerui64i_vNV(value,index,result):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei)
def glIndexFormatNV(type,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei)
def glNormalFormatNV(type,stride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei)
def glSecondaryColorFormatNV(size,type,stride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei)
def glTexCoordFormatNV(size,type,stride):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLsizei)
def glVertexAttribFormatNV(index,size,type,normalized,stride):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei)
def glVertexAttribIFormatNV(index,size,type,stride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei)
def glVertexFormatNV(size,type,stride):pass
